@@ -1,34 +1,60 @@
 /*
- * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999
- *	Ohio University.  All rights reserved.
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+ *	Ohio University.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that: (1) source code
- * distributions retain the above copyright notice and this paragraph
- * in its entirety, (2) distributions including binary code include
- * the above copyright notice and this paragraph in its entirety in
- * the documentation or other materials provided with the
- * distribution, and (3) all advertising materials mentioning features
- * or use of this software display the following acknowledgment:
- * ``This product includes software developed by the Ohio University
- * Internetworking Research Laboratory.''  Neither the name of the
- * University nor the names of its contributors may be used to endorse
- * or promote products derived from this software without specific
- * prior written permission.
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * ---
+ * 
+ * Starting with the release of tcptrace version 6 in 2001, tcptrace
+ * is licensed under the GNU General Public License (GPL).  We believe
+ * that, among the available licenses, the GPL will do the best job of
+ * allowing tcptrace to continue to be a valuable, freely-available
+ * and well-maintained tool for the networking community.
+ *
+ * Previous versions of tcptrace were released under a license that
+ * was much less restrictive with respect to how tcptrace could be
+ * used in commercial products.  Because of this, I am willing to
+ * consider alternate license arrangements as allowed in Section 10 of
+ * the GNU GPL.  Before I would consider licensing tcptrace under an
+ * alternate agreement with a particular individual or company,
+ * however, I would have to be convinced that such an alternative
+ * would be to the greater benefit of the networking community.
+ * 
+ * ---
+ *
+ * This file is part of Tcptrace.
+ *
+ * Tcptrace was originally written and continues to be maintained by
+ * Shawn Ostermann with the help of a group of devoted students and
+ * users (see the file 'THANKS').  The work on tcptrace has been made
+ * possible over the years through the generous support of NASA GRC,
+ * the National Science Foundation, and Sun Microsystems.
+ *
+ * Tcptrace is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Tcptrace is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Tcptrace (in the file 'COPYING'); if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+ * MA 02111-1307 USA
  * 
  * Author:	Shawn Ostermann
  * 		School of Electrical Engineering and Computer Science
  * 		Ohio University
  * 		Athens, OH
  *		ostermann@cs.ohiou.edu
+ *		http://www.tcptrace.org/
  */
 static char const copyright[] =
-    "@(#)Copyright (c) 1999 -- Shawn Ostermann -- Ohio University.  All rights reserved.\n";
+    "@(#)Copyright (c) 2001 -- Ohio University.\n";
 static char const rcsid[] =
-    "$Header: /home/sdo/src/tcptrace/src/RCS/compress.c,v 5.2 1999/02/25 15:01:26 sdo Exp $";
+    "$Header: /usr/local/cvs/tcptrace/compress.c,v 5.6 2002/06/21 09:56:26 alakhian Exp $";
 
 
 #include "tcptrace.h"
@@ -107,7 +133,7 @@ static char *FindBinary(
 	if (pch_colon)
 	    *pch_colon = '\00';
 
-	sprintf(abspath,"%s/%s",pch,binname);
+	snprintf(abspath,sizeof(abspath),"%s/%s",pch,binname);
 
 	if (debug>1)
 	    fprintf(stderr,"Checking for binary '%s'\n", abspath);
@@ -391,7 +417,11 @@ CompOpenPipe(
 	exit(-1);
     }
 
+#ifdef __VMS
+    child_pid = vfork();
+#else
     child_pid = fork();
+#endif
     if (child_pid == -1) {
 	perror("fork");
 	exit(-1);
@@ -441,6 +471,14 @@ CompOpenHeader(
     /* see if it's a supported compression file */
     pf = WhichFormat(filename);
 
+#ifdef __WIN32
+    if(pf != NULL) {
+       fprintf(stderr, "\nError: windows version of tcptrace does not support\nreading compressed dump files. Uncompress the file\nmanually and try again. Sorry!\n");
+       return((FILE *)-1);
+    }
+    return(NULL);
+#endif /* __WIN32 */   
+   
     /* if no compression found, just open the file */
     if (pf == NULL) {
 	if (freopen(filename,"r",stdin) == NULL) {
@@ -516,7 +554,11 @@ PipeHelper(void)
     }
     /* remember: fdpipe[0] is for reading, fdpipe[1] is for writing */
 
+#ifdef __VMS
+    child_pid = vfork();
+#else
     child_pid = fork();
+#endif
     if (child_pid == -1) {
 	perror("fork");
 	exit(-1);
