@@ -1,12 +1,68 @@
+/*
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+ *	Ohio University.
+ *
+ * ---
+ * 
+ * Starting with the release of tcptrace version 6 in 2001, tcptrace
+ * is licensed under the GNU General Public License (GPL).  We believe
+ * that, among the available licenses, the GPL will do the best job of
+ * allowing tcptrace to continue to be a valuable, freely-available
+ * and well-maintained tool for the networking community.
+ *
+ * Previous versions of tcptrace were released under a license that
+ * was much less restrictive with respect to how tcptrace could be
+ * used in commercial products.  Because of this, I am willing to
+ * consider alternate license arrangements as allowed in Section 10 of
+ * the GNU GPL.  Before I would consider licensing tcptrace under an
+ * alternate agreement with a particular individual or company,
+ * however, I would have to be convinced that such an alternative
+ * would be to the greater benefit of the networking community.
+ * 
+ * ---
+ *
+ * This file is part of Tcptrace.
+ *
+ * Tcptrace was originally written and continues to be maintained by
+ * Shawn Ostermann with the help of a group of devoted students and
+ * users (see the file 'THANKS').  The work on tcptrace has been made
+ * possible over the years through the generous support of NASA GRC,
+ * the National Science Foundation, and Sun Microsystems.
+ *
+ * Tcptrace is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Tcptrace is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Tcptrace (in the file 'COPYING'); if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+ * MA 02111-1307 USA
+ * 
+ * Author:	Shawn Ostermann
+ * 		School of Electrical Engineering and Computer Science
+ * 		Ohio University
+ * 		Athens, OH
+ *		ostermann@cs.ohiou.edu
+ *		http://www.tcptrace.org/
+ */
+static char const copyright[] =
+    "@(#)Copyright (c) 2001 -- Ohio University.\n";
+static char const rcsid[] =
+    "@(#)$Header: /usr/local/cvs/tcptrace/dyncounter.c,v 1.6 2001/05/31 21:26:42 sdo Exp $";
+
 /* dynamic counters/arrays */
 /* uses a 10-ary tree to manage a sparse counter space */
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
+#include "tcptrace.h"
 #include "dyncounter.h"
 
 
-static int debug = 0;
+static int ldebug = 0;
 
 /* external routines */
 void *MallocZ(int nbytes);
@@ -78,7 +134,7 @@ AddToCounter(
     u_long val,
     u_long granularity)
 {
-    if (debug)
+    if (ldebug)
 	fprintf(stderr,"AddToCounter(%p, %lu, %lu) called\n", *ppdc, ix, val);
 
     MakeCounter(ppdc,ix,val,granularity,0);
@@ -94,7 +150,7 @@ SetCounter(
     u_long val,
     u_long granularity)
 {
-    if (debug)
+    if (ldebug)
 	fprintf(stderr,"SetCounter(%p, %lu, %lu) called\n", *ppdc, ix, val);
 
     MakeCounter(ppdc,ix,val,granularity,1);
@@ -110,7 +166,7 @@ LookupCounter(
 {
     u_long *pdigit;
 
-    if (debug)
+    if (ldebug)
 	fprintf(stderr,"LookupCounter(p,%lu) called\n", ix);
 
     /* try to find the counter */
@@ -120,7 +176,7 @@ LookupCounter(
 
     if (pdigit == NULL) {
 	/* no leaf node == no such counter */
-	if (debug)
+	if (ldebug)
 	    fprintf(stderr,"LookupCounter(p,%lu): no such leaf\n", ix);
 	return(0);
     }
@@ -140,7 +196,7 @@ NextCounter(
     struct node *node;
     u_long nextix;
 
-    if (debug)
+    if (ldebug)
 	fprintf(stderr,"NextCounter(p,%p,%lu,%lu) called\n",
 		*((char **)pvoidcookie), *pix, *pcount);
 
@@ -156,7 +212,7 @@ NextCounter(
     /* make sure the linked list of leaves is up to date */
     if (pdc->firstleaf == NULL) {
 	FinishTree(pdc);
-	if (debug) {
+	if (ldebug) {
 	    PrintLeafList(pdc);
 	    PrintTree(pdc->tree);
 	}
@@ -309,7 +365,7 @@ FindLeaf(
     u_long valunits;
     u_long temp_value;
 
-    if (debug)
+    if (ldebug)
 	fprintf(stderr,"FindLeaf(%p(depth %lu), %lu, %s) called\n",
 		*ptree,
 		*ptree?((*ptree)->depth):0,
@@ -331,7 +387,7 @@ FindLeaf(
 	valunits *= CARDINALITY;
     }
 
-    if (debug)
+    if (ldebug)
 	fprintf(stderr,"FindLeaf: value:%lu  depth:%lu  units:%lu\n",
 		value, valdepth, valunits);
 
@@ -381,7 +437,7 @@ FindLeaf(
     hidigit = value / valunits;
     lowdigits = value % valunits;
 
-    if (debug)
+    if (ldebug)
 	fprintf(stderr,"for value %lu,  depth:%lu  units:%lu  hidigit:%lu  lowdigits:%lu\n",
 		value, valdepth, valunits, hidigit, lowdigits);
 
@@ -423,7 +479,7 @@ NewNode(
 {
     struct node *pn;
 
-    if (debug)
+    if (ldebug)
 	fprintf(stderr,"NewNode(%lu) called\n", depth);
 
     pn = MallocZ(sizeof(struct node));
@@ -441,7 +497,7 @@ FindTwig(
 {
     unsigned digit;
 
-    if (debug)
+    if (ldebug)
 	fprintf(stderr,"FindTwig(%p(depth %lu), %lu) called\n",
 		lastnode, lastnode->depth, ix);
 
@@ -465,7 +521,7 @@ FindCounter(
     struct node *pnode;
     u_long *pcounter;
 
-    if (debug)
+    if (ldebug)
 	fprintf(stderr,"FindCounter(p, %lu) called\n", ix);
 
     /* if the counter tree doesn't exist yet, create it */
@@ -480,7 +536,7 @@ FindCounter(
     if (ix < pdc->minix)
 	pdc->minix = ix;
 
-    if (debug>1)
+    if (ldebug>1)
 	PrintTree(pdc->tree);
 
     /* scale (TRUNCATE) the index by the granularity */
@@ -588,12 +644,12 @@ FinishTreeRecurse(
     if (pnode->depth == 1) {
 	if (pdc->firstleaf == NULL) {
 	    pdc->firstleaf = pdc->lastleaf = pnode;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,"FinishTree: Making %p the head\n", pnode);
 	} else {
 	    pdc->lastleaf->nextleaf = pnode;
 	    pdc->lastleaf = pnode;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,"FinishTree: Making %p the tail\n", pnode);
 	}
 	return;
@@ -630,7 +686,7 @@ NextCounterRecurse(
 {
     int i;
 
-    if (debug)
+    if (ldebug)
 	fprintf(stderr,"NextCounterRecurse(%p,%lu,%lu,%lu) called\n",
 		node, nextix, *pix, *pcount);
 
