@@ -1,32 +1,58 @@
 /*
- * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999
- *	Ohio University.  All rights reserved.
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+ *	Ohio University.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that: (1) source code
- * distributions retain the above copyright notice and this paragraph
- * in its entirety, (2) distributions including binary code include
- * the above copyright notice and this paragraph in its entirety in
- * the documentation or other materials provided with the
- * distribution, and (3) all advertising materials mentioning features
- * or use of this software display the following acknowledgment:
- * ``This product includes software developed by the Ohio University
- * Internetworking Research Laboratory.''  Neither the name of the
- * University nor the names of its contributors may be used to endorse
- * or promote products derived from this software without specific
- * prior written permission.
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * ---
+ * 
+ * Starting with the release of tcptrace version 6 in 2001, tcptrace
+ * is licensed under the GNU General Public License (GPL).  We believe
+ * that, among the available licenses, the GPL will do the best job of
+ * allowing tcptrace to continue to be a valuable, freely-available
+ * and well-maintained tool for the networking community.
+ *
+ * Previous versions of tcptrace were released under a license that
+ * was much less restrictive with respect to how tcptrace could be
+ * used in commercial products.  Because of this, I am willing to
+ * consider alternate license arrangements as allowed in Section 10 of
+ * the GNU GPL.  Before I would consider licensing tcptrace under an
+ * alternate agreement with a particular individual or company,
+ * however, I would have to be convinced that such an alternative
+ * would be to the greater benefit of the networking community.
+ * 
+ * ---
+ *
+ * This file is part of Tcptrace.
+ *
+ * Tcptrace was originally written and continues to be maintained by
+ * Shawn Ostermann with the help of a group of devoted students and
+ * users (see the file 'THANKS').  The work on tcptrace has been made
+ * possible over the years through the generous support of NASA GRC,
+ * the National Science Foundation, and Sun Microsystems.
+ *
+ * Tcptrace is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Tcptrace is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Tcptrace (in the file 'COPYING'); if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+ * MA 02111-1307 USA
  * 
  * Author:	Shawn Ostermann
  * 		School of Electrical Engineering and Computer Science
  * 		Ohio University
  * 		Athens, OH
  *		ostermann@cs.ohiou.edu
+ *		http://www.tcptrace.org/
  */
 static char const rcsid[] =
-   "$Header: /home/sdo/src/tcptrace/src/RCS/mod_rttgraph.c,v 5.3 1999/02/25 15:01:26 sdo Exp $";
+   "$Header: /usr/local/cvs/tcptrace/mod_rttgraph.c,v 5.7 2002/05/21 22:27:33 alakhian Exp $";
 
 #ifdef LOAD_MODULE_RTTGRAPH
 
@@ -81,7 +107,7 @@ static struct rttgraph_info {
 static struct rttgraph_info *MakeRttgraphRec();
 static void MakeBuckets(struct hist *phist, u_int num_buckets);
 static void AddSample(struct samples *psamp, u_short sample);
-static void PlotHist(FILE *f,struct hist *phist);
+static void PlotHist(MFILE *f,struct hist *phist);
 static void PlotOne(struct rttgraph_info *prttg);
 
 
@@ -222,7 +248,7 @@ rttgraph_read(
     u_long rtt_ms;
 
     /* find the start of the TCP header */
-    ptcp = (struct tcphdr *) ((char *)pip + 4*pip->ip_hl);
+    ptcp = (struct tcphdr *) ((char *)pip + 4*IP_HL(pip));
 
     /* make sure there we could have a RTT sample */
     if (!ACK_SET(ptcp))
@@ -262,7 +288,7 @@ DoHist(
 {
     int i;
     struct hist3d hist3d;
-    FILE *f;
+    MFILE *f;
     u_long sum;
     int slice;
     int base_z;
@@ -330,16 +356,16 @@ DoHist(
     }
 
 
-    if ((f = fopen("rtt.dat","w")) == NULL) {
+    if ((f = Mfopen("rtt.dat","w")) == NULL) {
 	perror("rtt.dat");
 	exit (1);
     }
     printf("Total Histogram\n");
     hist3d.rtt.z = -1;
     PlotHist(f,&hist3d.rtt);
-    fclose(f);
+    Mfclose(f);
 
-    if ((f = fopen("rtt3d.dat","w")) == NULL) {
+    if ((f = Mfopen("rtt3d.dat","w")) == NULL) {
 	perror("rtt.dat");
 	exit (1);
     }
@@ -367,14 +393,14 @@ DoHist(
     }
 #endif /* BROKEN */
 
-    fclose(f);
+    Mfclose(f);
 }
 
 
 
 static void
 PlotHist(
-    FILE *f,
+    MFILE *f,
     struct hist *phist)
 {
     int ms;
@@ -386,17 +412,20 @@ PlotHist(
     printf("  %lu samples\n", phist->num_samples);
     for (ms=0; ms < phist->num_buckets; ++ms) {
 	u_long count = phist->buckets[ms];
-	float percent = (float)count / phist->num_samples;
-	if (count == 0)
+	float percent;
+       
+	if (count == 0 || phist->num_samples == 0)
 	    continue;
 
+        percent = (float)count / phist->num_samples;
+       
 	printf("  %4d  %5lu  %5.2f\n", ms, count, 100 * percent);
 	if (z == -1)
-	    fprintf(f,"%4d  %.2f\n", ms, 100 * percent);
+	    Mfprintf(f,"%4d  %.2f\n", ms, 100 * percent);
 	else
-	    fprintf(f,"%4d  %d %.2f\n", ms, z, 100 * percent);
+	    Mfprintf(f,"%4d  %d %.2f\n", ms, z, 100 * percent);
     }
-    fprintf(f,"\n");
+    Mfprintf(f,"\n");
 }
 
 
