@@ -1,32 +1,58 @@
 /*
- * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999
- *	Ohio University.  All rights reserved.
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+ *	Ohio University.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that: (1) source code
- * distributions retain the above copyright notice and this paragraph
- * in its entirety, (2) distributions including binary code include
- * the above copyright notice and this paragraph in its entirety in
- * the documentation or other materials provided with the
- * distribution, and (3) all advertising materials mentioning features
- * or use of this software display the following acknowledgment:
- * ``This product includes software developed by the Ohio University
- * Internetworking Research Laboratory.''  Neither the name of the
- * University nor the names of its contributors may be used to endorse
- * or promote products derived from this software without specific
- * prior written permission.
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * ---
+ * 
+ * Starting with the release of tcptrace version 6 in 2001, tcptrace
+ * is licensed under the GNU General Public License (GPL).  We believe
+ * that, among the available licenses, the GPL will do the best job of
+ * allowing tcptrace to continue to be a valuable, freely-available
+ * and well-maintained tool for the networking community.
+ *
+ * Previous versions of tcptrace were released under a license that
+ * was much less restrictive with respect to how tcptrace could be
+ * used in commercial products.  Because of this, I am willing to
+ * consider alternate license arrangements as allowed in Section 10 of
+ * the GNU GPL.  Before I would consider licensing tcptrace under an
+ * alternate agreement with a particular individual or company,
+ * however, I would have to be convinced that such an alternative
+ * would be to the greater benefit of the networking community.
+ * 
+ * ---
+ *
+ * This file is part of Tcptrace.
+ *
+ * Tcptrace was originally written and continues to be maintained by
+ * Shawn Ostermann with the help of a group of devoted students and
+ * users (see the file 'THANKS').  The work on tcptrace has been made
+ * possible over the years through the generous support of NASA GRC,
+ * the National Science Foundation, and Sun Microsystems.
+ *
+ * Tcptrace is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Tcptrace is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Tcptrace (in the file 'COPYING'); if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+ * MA 02111-1307 USA
  * 
  * Author:	Shawn Ostermann
  * 		School of Electrical Engineering and Computer Science
  * 		Ohio University
  * 		Athens, OH
  *		ostermann@cs.ohiou.edu
+ *		http://www.tcptrace.org/
  */
 static char const rcsid[] =
-   "$Header: /home/sdo/src/tcptrace/src/RCS/mod_traffic.c,v 5.9 1999/07/12 22:10:58 sdo Exp $";
+   "$Header: /usr/local/cvs/tcptrace/mod_traffic.c,v 5.14 2001/08/01 20:47:59 mramadas Exp $";
 
 #ifdef LOAD_MODULE_TRAFFIC
 
@@ -216,7 +242,7 @@ static u_int rtt_maxvalid = 0xffffffff; /* maximum RTT to consider (ms) */
 
 
 /* local debugging flag */
-static int debug = 0;
+static int ldebug = 0;
 
 
 static void
@@ -239,7 +265,7 @@ ExcludePorts(
     CheckPortNum(firstport);
     CheckPortNum(lastport);
 
-    if (debug)
+    if (ldebug)
 	printf("mod_traffic: excluding ports [%d-%d]\n", firstport, lastport);
 
     while (firstport <= lastport)
@@ -255,7 +281,7 @@ IncludePorts(
     CheckPortNum(firstport);
     CheckPortNum(lastport);
 
-    if (debug)
+    if (ldebug)
 	printf("mod_traffic: including ports [%d-%d]\n", firstport, lastport);
 
     while (firstport <= lastport)
@@ -408,7 +434,7 @@ traffic_init(
 
     if (doplot_long) {
 	char title[100];
-	sprintf(title,"connections still open after %d seconds\n",
+	snprintf(title,sizeof(title),"connections still open after %d seconds\n",
 		longconn_duration);
 	plotter_long =
 	    new_plotter(NULL,
@@ -469,7 +495,7 @@ MakeTrafficRec(
 
     pti = MallocZ(sizeof(struct traffic_info));
 
-    if (debug>10)
+    if (ldebug>10)
 	printf("MakeTrafficRec(%d) called\n", (int)port);
 
     /* init */
@@ -543,7 +569,7 @@ traffic_read(
     void *plast,		/* past byte in the packet */
     void *mod_data)		/* connection info for this one */
 {
-    struct tcphdr *ptcp = (struct tcphdr *) ((char *)pip + 4*pip->ip_hl);
+    struct tcphdr *ptcp = (struct tcphdr *) ((char *)pip + 4*IP_HL(pip));
     struct traffic_info *pti1 = FindPort(ntohs(ptcp->th_sport));
     struct traffic_info *pti2 = FindPort(ntohs(ptcp->th_dport));
     u_long bytes = ntohs(pip->ip_len);
@@ -680,7 +706,7 @@ traffic_read(
 	    if ((rtt_min == -1) || (rtt_min > rtt))
 		rtt_min = rtt;
 
-	    if (debug > 9)
+	    if (ldebug > 9)
 		printf("Rtt: %d,  min:%d,  max:%d\n",
 		       rtt, rtt_min, rtt_max);
 	}
@@ -699,7 +725,7 @@ traffic_read(
     if (ACK_SET(ptcp)) {
 	int tcp_length, tcp_data_length;
 	tcp_length = getpayloadlength(pip, plast);
-	tcp_data_length = tcp_length - (4 * ptcp->th_off);
+	tcp_data_length = tcp_length - (4 * TH_OFF(ptcp));
 	if (tcp_data_length == 0) {
 	    if (pti1) {
 		++pti1->npureacks;
@@ -728,7 +754,7 @@ PortName(
     if (port == 0)
 	return("total");
 
-    sprintf(buf,"%d",port);
+    snprintf(buf,sizeof(buf),"%d",port);
     return(buf);
 }
 
@@ -750,7 +776,7 @@ AgeTraffic(void)
 
     /* check elapsed time */
     etime = elapsed(last_time, current_time);
-    if (debug>1)
+    if (ldebug>1)
 	printf("AgeTraffic called, elapsed time is %.3f seconds\n", etime/1000000);
     if (etime == 0.0)
 	return;
@@ -867,7 +893,7 @@ AgeTraffic(void)
     /* ============================================================ */
     /* print them out */
     for (pti=traffichead; pti; pti=pti->next) {
-	if (debug>1)
+	if (ldebug>1)
 	    printf("  Aging Port %u   bytes: %lu  packets: %lu\n",
 		   pti->port, pti->nbytes, pti->npackets);
 
@@ -996,17 +1022,17 @@ bytes: %12lu  pkts: %10lu  conns: %8lu  tput: %8lu B/s\n",
 	     etime_secs, elapsed2str(etime));
 
     /* ttl bytes */
-    Mfprintf(pmf, "%llu ttl bytes sent, %.3f bytes/second\n",
+    Mfprintf(pmf, "%" FS_ULL " ttl bytes sent, %.3f bytes/second\n",
 	     data_nbytes_all,
 	     (float)data_nbytes_all / ((float)etime_secs));
 
     /* ttl bytes (nonrexmit)*/
-    Mfprintf(pmf, "%llu ttl non-rexmit bytes sent, %.3f bytes/second\n",
+    Mfprintf(pmf, "%" FS_ULL " ttl non-rexmit bytes sent, %.3f bytes/second\n",
 	     data_nbytes_nonrexmit,
 	     (float)data_nbytes_nonrexmit / ((float)etime_secs));
 
     /* ttl bytes (nonrexmit)*/
-    Mfprintf(pmf, "%llu ttl rexmit bytes sent, %.3f bytes/second\n",
+    Mfprintf(pmf, "%" FS_ULL " ttl rexmit bytes sent, %.3f bytes/second\n",
 	     data_nbytes_all - data_nbytes_nonrexmit,
 	     (float)(data_nbytes_all - data_nbytes_nonrexmit) /
 	     ((float)etime_secs));
@@ -1116,13 +1142,13 @@ ParseArgs(char *argstring)
     /* check the module args */
     for (i=1; i < argc; ++i) {
 	float interval;
-	if (debug > 1)
+	if (ldebug > 1)
 	    printf("Checking argv[%d]: '%s'\n", i, argv[i]);
 	if (strcmp(argv[i],"-d") == 0) {
-	    ++debug;
+	    ++ldebug;
 	} else if (sscanf(argv[i],"-i%f", &interval) == 1) {
 	    age_interval = interval;
-	    if (debug)
+	    if (ldebug)
 		printf("mod_traffic: setting age interval to %.3f seconds\n",
 		       age_interval);
 	} else if (strcmp(argv[i],"-G") == 0) {
@@ -1138,48 +1164,48 @@ ParseArgs(char *argstring)
 	    doplot_i_open = TRUE;
 	    doplot_packets = TRUE;
 	    doplot_pureacks = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating all graphs\n");
 	} else if (strcmp(argv[i],"-A") == 0) {
 	    doplot_active = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'active' graph into '%s'\n",
 			PLOTTER_ACTIVE_FILENAME);
 	} else if (strcmp(argv[i],"-B") == 0) {
 	    doplot_bytes = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'bytes' graph into '%s'\n",
 			PLOTTER_BYTES_FILENAME);
 	} else if (strcmp(argv[i],"-H") == 0) {
 	    doplot_halfopen = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'halfopen' graph into '%s'\n",
 			PLOTTER_HALFOPEN_FILENAME);
 	} else if (strcmp(argv[i],"-Q") == 0) {
 	    doplot_idle = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'idle' graph into '%s'\n",
 			PLOTTER_IDLE_FILENAME);
 	} else if (strcmp(argv[i],"-K") == 0) {
 	    doplot_pureacks = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'pureacks' graph into '%s'\n",
 			PLOTTER_PUREACKS_FILENAME);
 	} else if (strcmp(argv[i],"-L") == 0) {
 	    doplot_loss = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'loss' graph into '%s'\n",
 			PLOTTER_LOSS_FILENAME);
 	} else if (strcmp(argv[i],"-T") == 0) {
 	    doplot_data = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'total data' graph into '%s'\n",
 			PLOTTER_DATA_FILENAME);
@@ -1195,39 +1221,39 @@ ParseArgs(char *argstring)
 		}
 
 	    }
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'long duration' graph (%d secs) into '%s'\n",
 			longconn_duration,
 			PLOTTER_LONG_FILENAME);
 	} else if (strcmp(argv[i],"-O") == 0) {
 	    doplot_open = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'open' graph into '%s'\n",
 			PLOTTER_OPEN_FILENAME);
 	} else if (strcmp(argv[i],"-C") == 0) {
 	    doplot_openclose = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'openclose' graph into '%s'\n",
 			PLOTTER_OPENCLOSE_FILENAME);
 	} else if (strcmp(argv[i],"-I") == 0) {
 	    doplot_i_open = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'instantaneous openclose' graph into '%s'\n",
 			PLOTTER_I_OPEN_FILENAME);
 	} else if (strcmp(argv[i],"-P") == 0) {
 	    doplot_packets = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'packets' graph into '%s'\n",
 			PLOTTER_PACKETS_FILENAME);
 	} else if (strncmp(argv[i],"-R",2) == 0) {
 	    int nargs;
 	    doplot_rtt = TRUE;
-	    if (debug)
+	    if (ldebug)
 		fprintf(stderr,
 			"mod_traffic: generating 'rtt' graph into '%s'\n",
 			PLOTTER_RTT_FILENAME);
