@@ -1,32 +1,58 @@
 /*
- * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999
- *	Ohio University.  All rights reserved.
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+ *	Ohio University.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that: (1) source code
- * distributions retain the above copyright notice and this paragraph
- * in its entirety, (2) distributions including binary code include
- * the above copyright notice and this paragraph in its entirety in
- * the documentation or other materials provided with the
- * distribution, and (3) all advertising materials mentioning features
- * or use of this software display the following acknowledgment:
- * ``This product includes software developed by the Ohio University
- * Internetworking Research Laboratory.''  Neither the name of the
- * University nor the names of its contributors may be used to endorse
- * or promote products derived from this software without specific
- * prior written permission.
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * ---
+ * 
+ * Starting with the release of tcptrace version 6 in 2001, tcptrace
+ * is licensed under the GNU General Public License (GPL).  We believe
+ * that, among the available licenses, the GPL will do the best job of
+ * allowing tcptrace to continue to be a valuable, freely-available
+ * and well-maintained tool for the networking community.
+ *
+ * Previous versions of tcptrace were released under a license that
+ * was much less restrictive with respect to how tcptrace could be
+ * used in commercial products.  Because of this, I am willing to
+ * consider alternate license arrangements as allowed in Section 10 of
+ * the GNU GPL.  Before I would consider licensing tcptrace under an
+ * alternate agreement with a particular individual or company,
+ * however, I would have to be convinced that such an alternative
+ * would be to the greater benefit of the networking community.
+ * 
+ * ---
+ *
+ * This file is part of Tcptrace.
+ *
+ * Tcptrace was originally written and continues to be maintained by
+ * Shawn Ostermann with the help of a group of devoted students and
+ * users (see the file 'THANKS').  The work on tcptrace has been made
+ * possible over the years through the generous support of NASA GRC,
+ * the National Science Foundation, and Sun Microsystems.
+ *
+ * Tcptrace is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Tcptrace is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Tcptrace (in the file 'COPYING'); if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+ * MA 02111-1307 USA
  * 
  * Author:	Shawn Ostermann
  * 		School of Electrical Engineering and Computer Science
  * 		Ohio University
  * 		Athens, OH
  *		ostermann@cs.ohiou.edu
+ *		http://www.tcptrace.org/
  */
 static char const rcsid_modules[] =
-    "$Header: /home/sdo/src/tcptrace/src/RCS/modules.h,v 5.2 1999/02/25 15:01:37 sdo Exp $";
+    "$Header: /usr/local/cvs/tcptrace/modules.h,v 5.5 2001/05/31 20:55:02 sdo Exp $";
 
 
 /* 
@@ -102,6 +128,12 @@ struct module {
     void (*module_nontcpudp_read) (
 	struct ip *pip,		/* the packet */
 	void *plast);		/* pointer to last byte */
+
+    /* Called for old TCP connections when they are deleted by */
+    /* the real-time version of the program */ 
+    void (*module_deleteconn) (
+	 tcp_pair *ptp,		/* info I have about this connection */
+	 void *pmodstruct);	/* module-specific structure */
 };
 
 
@@ -130,6 +162,16 @@ struct module {
 #endif /* LOAD_MODULE_COLLIE */
 
 
+#ifdef LOAD_MODULE_SLICE
+#include "mod_slice.h"	/* for the slice package */
+#endif /* LOAD_MODULE_SLICE */
+
+
+#ifdef LOAD_MODULE_REALTIME
+#include "mod_realtime.h"       /* for the example real-time package */
+#endif /* LOAD_MODULE_REALTIME */
+
+
 /* declare (install) the various module routines */
 struct module modules[] = {
 #ifdef LOAD_MODULE_HTTP
@@ -143,7 +185,7 @@ struct module modules[] = {
      http_usage,		/* routine to call to print module usage */
      http_newfile,		/* routine to call on each new file */
      http_newconn,		/* routine to call on each new connection */
-     NULL,NULL,NULL},			/* not interested in non-tcp */
+     NULL, NULL, NULL, NULL},	/* not interested in non-tcp */
 #endif /* LOAD_MODULE_HTTP */
 
     /* list other modules here ... */
@@ -158,7 +200,7 @@ struct module modules[] = {
      tcplib_usage,		/* routine to call to print module usage */
      tcplib_newfile,		/* routine to call on each new file */
      tcplib_newconn,		/* routine to call on each new connection */
-     NULL,NULL,NULL},			/* not interested in non-tcp */
+     NULL, NULL, NULL, NULL},	/* not interested in non-tcp */
 #endif /* LOAD_MODULE_TCPLIB */
 
 
@@ -167,9 +209,17 @@ struct module modules[] = {
     {TRUE,			/* make FALSE if you don't want to call it at all */
      "traffic", "traffic analysis package",
      traffic_init, traffic_read, traffic_done,		
-     traffic_usage, NULL, traffic_newconn, NULL,NULL,NULL},
+     traffic_usage, NULL, traffic_newconn, NULL, NULL, NULL, NULL},
 #endif /* LOAD_MODULE_TRAFFIC */
 
+#ifdef LOAD_MODULE_SLICE
+    /* ttl slice analysis */
+    {TRUE,			/* make FALSE if you don't want to call it at all */
+     "slice", "traffic efficiency data by time slices",
+     slice_init, slice_read, slice_done,		
+     slice_usage, NULL, slice_newconn, NULL, NULL, NULL, NULL},
+#endif /* LOAD_MODULE_SLICE */
+
 #ifdef LOAD_MODULE_RTTGRAPH
     {TRUE,			/* make FALSE if you don't want to call it at all */
      "rttgraph", "round trip time analysis graphs",
@@ -179,7 +229,7 @@ struct module modules[] = {
      rttgraph_usage,		/* routine to call to print module usage */
      NULL,			/* routine to call on each new file */
      rttgraph_newconn,		/* routine to call on each new connection */
-     NULL,NULL,NULL},			/* not interested in non-tcp */
+     NULL, NULL, NULL, NULL},	/* not interested in non-tcp */
 #endif /* LOAD_MODULE_TRAFFIC */
 
 #ifdef LOAD_MODULE_COLLIE
@@ -188,7 +238,24 @@ struct module modules[] = {
      "collie", "connection summary package",
      collie_init, NULL /* read */, collie_done,		
      collie_usage, collie_newfile, collie_newconn,
-     NULL, collie_newudpconn, NULL},
+     NULL, collie_newudpconn, NULL, NULL},
 #endif /* LOAD_MODULE_COLLIE */
+
+#ifdef LOAD_MODULE_REALTIME
+    {TRUE,		         /* make FALSE if you don't want to call it at all */
+     "realtime",                 /* name of the module */
+     "example real-time package",/* description of the module */
+     realtime_init,		 /* routine to call to init the module */
+     realtime_read,		 /* routine to pass each TCP segment */
+     realtime_done,		 /* routine to call at program end */
+     realtime_usage,		 /* routine to call to print module usage */
+     NULL,			 /* routine to call on each new file */
+     realtime_newconn,		 /* routine to call on each new connection */
+     realtime_udp_read,          /* routine to pass each UDP segment */
+     NULL,              	 /* routine to call on each new UDP conn */
+     realtime_nontcpudp_read, 	 /* routine to pass each non-tcp and non-udp 
+				    packets*/
+     realtime_deleteconn},
+#endif /* LOAD_MODULE_REALTIME */
 };
 #define NUM_MODULES (sizeof(modules) / sizeof(struct module))
