@@ -1,34 +1,60 @@
 /*
- * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999
- *	Ohio University.  All rights reserved.
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+ *	Ohio University.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that: (1) source code
- * distributions retain the above copyright notice and this paragraph
- * in its entirety, (2) distributions including binary code include
- * the above copyright notice and this paragraph in its entirety in
- * the documentation or other materials provided with the
- * distribution, and (3) all advertising materials mentioning features
- * or use of this software display the following acknowledgment:
- * ``This product includes software developed by the Ohio University
- * Internetworking Research Laboratory.''  Neither the name of the
- * University nor the names of its contributors may be used to endorse
- * or promote products derived from this software without specific
- * prior written permission.
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * ---
+ * 
+ * Starting with the release of tcptrace version 6 in 2001, tcptrace
+ * is licensed under the GNU General Public License (GPL).  We believe
+ * that, among the available licenses, the GPL will do the best job of
+ * allowing tcptrace to continue to be a valuable, freely-available
+ * and well-maintained tool for the networking community.
+ *
+ * Previous versions of tcptrace were released under a license that
+ * was much less restrictive with respect to how tcptrace could be
+ * used in commercial products.  Because of this, I am willing to
+ * consider alternate license arrangements as allowed in Section 10 of
+ * the GNU GPL.  Before I would consider licensing tcptrace under an
+ * alternate agreement with a particular individual or company,
+ * however, I would have to be convinced that such an alternative
+ * would be to the greater benefit of the networking community.
+ * 
+ * ---
+ *
+ * This file is part of Tcptrace.
+ *
+ * Tcptrace was originally written and continues to be maintained by
+ * Shawn Ostermann with the help of a group of devoted students and
+ * users (see the file 'THANKS').  The work on tcptrace has been made
+ * possible over the years through the generous support of NASA GRC,
+ * the National Science Foundation, and Sun Microsystems.
+ *
+ * Tcptrace is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Tcptrace is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Tcptrace (in the file 'COPYING'); if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+ * MA 02111-1307 USA
  * 
  * Author:	Shawn Ostermann
  * 		School of Electrical Engineering and Computer Science
  * 		Ohio University
  * 		Athens, OH
  *		ostermann@cs.ohiou.edu
+ *		http://www.tcptrace.org/
  */
 static char const copyright[] =
-    "@(#)Copyright (c) 1999 -- Shawn Ostermann -- Ohio University.  All rights reserved.\n";
+    "@(#)Copyright (c) 2001 -- Ohio University.\n";
 static char const rcsid[] =
-    "@(#)$Header: /home/sdo/src/tcptrace/src/RCS/output.c,v 5.3 1999/02/25 15:01:26 sdo Exp $";
+    "@(#)$Header: /usr/local/cvs/tcptrace/output.c,v 5.18 2002/05/22 20:33:08 alakhian Exp $";
 
 
 #include "tcptrace.h"
@@ -155,7 +181,7 @@ FormatBrief(
     tcb *pba = &ptp->b2a;
     static char infobuf[100];
 
-    sprintf(infobuf,"%s - %s (%s2%s)",
+    snprintf(infobuf,sizeof(infobuf),"%s - %s (%s2%s)",
 	    ptp->a_endpoint, ptp->b_endpoint,
 	    pab->host_letter, pba->host_letter);
     return(infobuf);
@@ -180,10 +206,15 @@ PrintTrace(
     char *host2 = pba->host_letter;
     char bufl[40],bufr[40];
 
+    /* counters to use for seq. space wrap around calculations
+     */
+    u_llong stream_length_pab=0, stream_length_pba=0;
+    u_long pab_last, pba_last;
+    
     fprintf(stdout,"\thost %-4s      %s\n",
-	    (sprintf(bufl,"%s:", host1),bufl), ptp->a_endpoint);
+	    (snprintf(bufl,sizeof(bufl),"%s:", host1),bufl), ptp->a_endpoint);
     fprintf(stdout,"\thost %-4s      %s\n",
-	    (sprintf(bufl,"%s:", host2),bufl), ptp->b_endpoint);
+	    (snprintf(bufl,sizeof(bufl),"%s:", host2),bufl), ptp->b_endpoint);
     fprintf(stdout,"\tcomplete conn: %s",
 	    ConnReset(ptp)?"RESET":(
 		ConnComplete(ptp)?"yes":"no"));
@@ -201,11 +232,7 @@ PrintTrace(
     etime_usecs = 1000000 * (etime/1000000.0 - (double)etime_secs);
     fprintf(stdout,"\telapsed time:  %s\n", elapsed2str(etime));
 
-#ifdef HAVE_LONG_LONG
-    fprintf(stdout,"\ttotal packets: %llu\n", ptp->packets);
-#else  /* HAVE_LONG_LONG */
-    fprintf(stdout,"\ttotal packets: %lu\n", ptp->packets);
-#endif  /* HAVE_LONG_LONG */
+    fprintf(stdout,"\ttotal packets: %" FS_ULL "\n", ptp->packets);
 
     fprintf(stdout,"\tfilename:      %s\n", ptp->filename);
 
@@ -217,6 +244,8 @@ PrintTrace(
 	StatLineI("resets sent","", pab->reset_count, pba->reset_count);
     StatLineI("ack pkts sent","", pab->ack_pkts, pba->ack_pkts);
     StatLineI("pure acks sent","", pab->pureack_pkts, pba->pureack_pkts);
+    StatLineI("sack pkts sent","", pab->num_sacks, pba->num_sacks);
+    StatLineI("max sack blks/ack","", pab->max_sack_blocks, pba->max_sack_blocks);
     StatLineI("unique bytes sent","",
 	      pab->unique_bytes, pba->unique_bytes);
     StatLineI("actual data pkts","", pab->data_pkts, pba->data_pkts);
@@ -224,19 +253,23 @@ PrintTrace(
     StatLineI("rexmt data pkts","", pab->rexmit_pkts, pba->rexmit_pkts);
     StatLineI("rexmt data bytes","",
 	      pab->rexmit_bytes, pba->rexmit_bytes);
+    StatLineI("zwnd probe pkts","", 
+		  pab->num_zwnd_probes, pba->num_zwnd_probes);
+    StatLineI("zwnd probe bytes","",
+	      pab->zwnd_probe_bytes, pba->zwnd_probe_bytes);
     StatLineI("outoforder pkts","",
 	      pab->out_order_pkts, pba->out_order_pkts);
     StatLineI("pushed data pkts","", pab->data_pkts_push, pba->data_pkts_push);
     StatLineP("SYN/FIN pkts sent","","%s",
-	      (sprintf(bufl,"%d/%d",
+	      (snprintf(bufl,sizeof(bufl),"%d/%d",
 		       pab->syn_count, pab->fin_count),bufl),
-	      (sprintf(bufr,"%d/%d",
+	      (snprintf(bufr,sizeof(bufr),"%d/%d",
 		       pba->syn_count, pba->fin_count),bufr));
     if (pab->f1323_ws || pba->f1323_ws || pab->f1323_ts || pba->f1323_ts) {
 	StatLineP("req 1323 ws/ts","","%s",
-		  (sprintf(bufl,"%c/%c",
+		  (snprintf(bufl,sizeof(bufl),"%c/%c",
 		      pab->f1323_ws?'Y':'N',pab->f1323_ts?'Y':'N'),bufl),
-		  (sprintf(bufr,"%c/%c",
+		  (snprintf(bufr,sizeof(bufr),"%c/%c",
 		      pba->f1323_ws?'Y':'N',pba->f1323_ts?'Y':'N'),bufr));
     }
     if (pab->f1323_ws || pba->f1323_ws) {
@@ -251,6 +284,12 @@ PrintTrace(
 		  pab->sacks_sent,
 		  pba->sacks_sent);
     }
+    StatLineI("urgent data pkts", "pkts",
+	      pab->urg_data_pkts,
+	      pba->urg_data_pkts);
+    StatLineI("urgent data bytes", "bytes",
+	      pab->urg_data_bytes,
+	      pba->urg_data_bytes);
     StatLineI("mss requested","bytes", pab->mss, pba->mss);
     StatLineI("max segm size","bytes",
 	      pab->max_seg_size,
@@ -265,38 +304,119 @@ PrintTrace(
     StatLineI("min win adv","bytes", pab->win_min, pba->win_min);
     StatLineI("zero win adv","times",
 	      pab->win_zero_ct, pba->win_zero_ct);
+    /* Changed this computation from
+     *   division by pXX->ack_pkts to
+     *   division by pXX->packets
+     * --Avinash.
+     */ 
     StatLineI("avg win adv","bytes",
-	      pab->ack_pkts==0?0:pab->win_tot/pab->ack_pkts,
-	      pba->ack_pkts==0?0:pba->win_tot/pba->ack_pkts);
-    if (print_cwin) {
-	StatLineI("max cwin","bytes", pab->cwin_max, pba->cwin_max);
-	StatLineI("min cwin","bytes", pab->cwin_min, pba->cwin_min);
-	StatLineI("avg cwin","bytes",
-		  pab->ack_pkts==0?0:pab->cwin_tot/pab->ack_pkts,
-		  pba->ack_pkts==0?0:pba->cwin_tot/pba->ack_pkts);
+	      pab->ack_pkts==0?0:pab->win_tot/pab->packets,
+	      pba->ack_pkts==0?0:pba->win_tot/pba->packets);
+    if (print_owin) {
+	StatLineI("max owin","bytes", pab->owin_max, pba->owin_max);
+	StatLineI("min non-zero owin","bytes", pab->owin_min, pba->owin_min);
+	StatLineI("avg owin","bytes",
+		  pab->owin_count==0?0:pab->owin_tot/pab->owin_count,
+		  pba->owin_count==0?0:pba->owin_tot/pba->owin_count);
+	if (etime == 0.0) {
+		StatLineP("wavg owin", "", "%s", "NA", "NA");	
+	}
+	else {
+		StatLineI("wavg owin","bytes", 
+			  (u_llong)(pab->owin_wavg/((double)etime/1000000)), 
+		  	  (u_llong)(pba->owin_wavg/((double)etime/1000000)));
+   	} 
     }
     StatLineI("initial window","bytes",
 	      pab->initialwin_bytes, pba->initialwin_bytes);
     StatLineI("initial window","pkts",
 	      pab->initialwin_segs, pba->initialwin_segs);
 
-
     /* compare to theoretical length of the stream (not just what
        we saw) using the SYN and FIN
-       (N.B. not taking wrapped seq space into account) */
-    if ((pab->syn_count > 0) && (pab->fin_count > 0) &&
-	(pba->syn_count > 0) && (pba->fin_count > 0)) {
-	StatLineI("ttl stream length","bytes",
-		  pab->fin-pab->syn-1,
-		  pba->fin-pba->syn-1);
-	StatLineI("missed data","bytes",
-		  pab->fin-pab->syn-1-pab->unique_bytes,
-		  pba->fin-pba->syn-1-pba->unique_bytes);
-    } else {
-	StatLineP("ttl stream length","","%s","NA","NA");
-	StatLineP("missed data","","%s","NA","NA");
+     * Seq. Space wrap around calculations:
+     * Calculate stream length using last_seq_num seen, first_seq_num
+     * seen and wrap_count.
+     * first_seq_num = syn
+     * If reset_set, last_seq_num = latest_seq
+     *          else last_seq_num = fin
+     */
+    
+    pab_last = (pab->reset_count>0)?pab->latest_seq:pab->fin;
+    
+    pba_last = (pba->reset_count>0)?pba->latest_seq:pba->fin;
+    
+    /* calculating stream length for direction pab */
+    if ((pab->syn_count > 0) && (pab->fin_count > 0)) {
+	if (pab->seq_wrap_count > 0) {
+	    if (pab_last > pab->syn) {
+		stream_length_pab = pab_last + (MAX_32 * pab->seq_wrap_count) - pab->syn - 1;
+	    }
+	    else {
+		stream_length_pab = pab_last + (MAX_32 * (pab->seq_wrap_count+1)) - pab->syn - 1;
+	    }
+	}
+	else {
+	    if (pab_last > pab->syn) {
+		stream_length_pab = pab_last - pab->syn - 1;
+	    }
+	    else {
+		stream_length_pab = MAX_32 + pab_last - pab->syn - 1;
+	    }
+	}
     }
 
+    /* calculating stream length for direction pba */
+    if ((pba->syn_count > 0) && (pba->fin_count > 0)) {
+	if (pba->seq_wrap_count > 0) {
+	    if (pba_last > pba->syn) {
+		stream_length_pba = pba_last + (MAX_32 * pba->seq_wrap_count) - pba->syn - 1;
+	    }
+	    else {
+		stream_length_pba = pba_last + (MAX_32 * (pba->seq_wrap_count+1)) - pba->syn - 1;
+	    }
+	}
+	else {
+	    if (pba_last > pba->syn) {
+		stream_length_pba = pba_last - pba->syn - 1;
+	    }
+	    else {
+		stream_length_pba = MAX_32 + pba_last - pba->syn - 1;
+	    }
+	}
+    }
+
+    /* print out values */
+    if ((pab->fin_count > 0) && (pab->syn_count > 0)) {
+	char *format = "%8" FS_ULL;
+	StatLineFieldL("ttl stream length", "bytes", format, stream_length_pab, 0);
+    }
+    else {
+	StatLineField("ttl stream length", "", "%s", (u_long)"NA", 0);
+    }
+    if ((pba->fin_count > 0) && (pba->syn_count > 0)) {
+	char *format = "%8" FS_ULL;
+	StatLineFieldL("ttl stream length", "bytes", format, stream_length_pba, 1);
+    }
+    else {
+	StatLineField("ttl stream length", "", "%s", (u_long)"NA", 1);
+    }
+
+    if ((pab->fin_count > 0) && (pab->syn_count > 0)) {
+	char *format = "%8" FS_ULL;
+	StatLineFieldL("missed data", "bytes", format, (stream_length_pab - pab->unique_bytes), 0);
+    }
+    else {
+	StatLineField("missed data", "", "%s", (u_long)"NA", 0);
+    }
+    if ((pba->fin_count > 0) && (pba->syn_count > 0)) {
+	char *format = "%8" FS_ULL;
+	StatLineFieldL("missed data", "bytes", format, (stream_length_pba - pba->unique_bytes), 1);
+    }
+    else {
+	StatLineField("missed data", "", "%s", (u_long)"NA", 1);
+    }
+    
     /* tell how much data was NOT captured in the files */
     StatLineI("truncated data","bytes",
 	      pab->trunc_bytes, pba->trunc_bytes);
@@ -314,9 +434,9 @@ PrintTrace(
 	      etime_data2 / 1000000.0);
     StatLineP("idletime max","ms","%s",
 	      ZERO_TIME(&pab->last_time)?"NA":
-	      (sprintf(bufl,"%8.1f",(double)pab->idle_max/1000.0),bufl),
+	      (snprintf(bufl,sizeof(bufl),"%8.1f",(double)pab->idle_max/1000.0),bufl),
 	      ZERO_TIME(&pba->last_time)?"NA":
-	      (sprintf(bufr,"%8.1f",(double)pba->idle_max/1000.0),bufr));
+	      (snprintf(bufr,sizeof(bufr),"%8.1f",(double)pba->idle_max/1000.0),bufr));
 
     if ((pab->num_hardware_dups != 0) || (pba->num_hardware_dups != 0)) {
 	StatLineI("hardware dups","segs",
@@ -349,6 +469,26 @@ PrintTrace(
 	StatLineF("RTT stdev","ms","%8.1f",
 		  Stdev(pab->rtt_sum, pab->rtt_sum2, pab->rtt_count) / 1000.0,
 		  Stdev(pba->rtt_sum, pba->rtt_sum2, pba->rtt_count) / 1000.0);
+	fprintf(stdout,"\n");
+	StatLineF("RTT from 3WHS","ms","%8.1f",
+		  (double)pab->rtt_3WHS/1000.0,
+		  (double)pba->rtt_3WHS/1000.0);
+	fprintf(stdout,"\n");
+	StatLineI("RTT full_sz smpls","", 
+		  pab->rtt_full_count, pba->rtt_full_count);
+	StatLineF("RTT full_sz min","ms","%8.1f",
+		  (double)pab->rtt_full_min/1000.0,
+		  (double)pba->rtt_full_min/1000.0);
+	StatLineF("RTT full_sz max","ms","%8.1f",
+		  (double)pab->rtt_full_max/1000.0,
+		  (double)pba->rtt_full_max/1000.0);
+	StatLineF("RTT full_sz avg","ms","%8.1f",
+		  Average(pab->rtt_full_sum, pab->rtt_full_count) / 1000.0,
+		  Average(pba->rtt_full_sum, pba->rtt_full_count) / 1000.0);
+	StatLineF("RTT full_sz stdev","ms","%8.1f",
+		  Stdev(pab->rtt_full_sum, pab->rtt_full_sum2, pab->rtt_full_count) / 1000.0,
+		  Stdev(pba->rtt_full_sum, pba->rtt_full_sum2, pba->rtt_full_count) / 1000.0);
+	fprintf(stdout,"\n");
 
 	StatLineI("post-loss acks","",
 		  pab->rtt_nosample, pba->rtt_nosample);
@@ -435,31 +575,22 @@ PrintBrief(
     if (TRUE) {
 	/* new version */
 	fprintf(stdout,"%*s", -max_width, FormatBrief(ptp));
-#ifdef HAVE_LONG_LONG
-	fprintf(stdout," %4llu>", pab->packets);
-	fprintf(stdout," %4llu<", pba->packets);
-#else /* HAVE_LONG_LONG */
-	fprintf(stdout," %4lu>", pab->packets);
-	fprintf(stdout," %4lu<", pba->packets);
-#endif /* HAVE_LONG_LONG */
+
+	fprintf(stdout," %4" FS_ULL ">", pab->packets);
+	fprintf(stdout," %4" FS_ULL "<", pba->packets);
+
     } else {
 	/* old version */
 	fprintf(stdout,"%s <==> %s",
 		ptp->a_endpoint,
 		ptp->b_endpoint);
-#ifdef HAVE_LONG_LONG
-	fprintf(stdout,"  %s2%s:%llu",
-#else /* HAVE_LONG_LONG */
-	fprintf(stdout,"  %s2%s:%lu",
-#endif /* HAVE_LONG_LONG */
+
+	fprintf(stdout,"  %s2%s:%"FS_ULL,
 		pab->host_letter,
 		pba->host_letter,
 		pab->packets);
-#ifdef HAVE_LONG_LONG
-	fprintf(stdout,"  %s2%s:%llu",
-#else /* HAVE_LONG_LONG */
-	fprintf(stdout,"  %s2%s:%lu",
-#endif /* HAVE_LONG_LONG */
+
+	fprintf(stdout,"  %s2%s:%"FS_ULL,
 		pba->host_letter,
 		pab->host_letter,
 		pba->packets);
@@ -505,9 +636,9 @@ UDPPrintTrace(
     char bufl[40];
 
     fprintf(stdout,"\thost %-4s      %s\n",
-	    (sprintf(bufl,"%s:", host1),bufl), pup->a_endpoint);
+	    (snprintf(bufl,sizeof(bufl),"%s:", host1),bufl), pup->a_endpoint);
     fprintf(stdout,"\thost %-4s      %s\n",
-	    (sprintf(bufl,"%s:", host2),bufl), pup->b_endpoint);
+	    (snprintf(bufl,sizeof(bufl),"%s:", host2),bufl), pup->b_endpoint);
     fprintf(stdout,"\n");
 
     fprintf(stdout,"\tfirst packet:  %s\n", ts2ascii(&pup->first_time));
@@ -518,11 +649,7 @@ UDPPrintTrace(
     etime_usecs = 1000000 * (etime/1000000.0 - (double)etime_secs);
     fprintf(stdout,"\telapsed time:  %s\n", elapsed2str(etime));
 
-#ifdef HAVE_LONG_LONG
-    fprintf(stdout,"\ttotal packets: %llu\n", pup->packets);
-#else  /* HAVE_LONG_LONG */
-    fprintf(stdout,"\ttotal packets: %lu\n", pup->packets);
-#endif  /* HAVE_LONG_LONG */
+    fprintf(stdout,"\ttotal packets: %" FS_ULL "\n", pup->packets);
 
     fprintf(stdout,"\tfilename:      %s\n", pup->filename);
 
@@ -581,7 +708,7 @@ StatLineI_LL(
     u_llong argleft,
     u_llong argright)
 {
-    char *format = "%8llu";
+    char *format = "%8" FS_ULL;
     StatLineFieldL(label,units,format,argleft,0);
     StatLineFieldL(label,units,format,argright,1);
 }
@@ -594,10 +721,14 @@ StatLineFieldL(
     u_llong arg,
     int	f_rightside)
 {
-    char valbuf[20];
+    /* bug fix: Theo Snelleman */
+    /* "The biggest number possible is 18446744073709551615 (20 digits) and
+        is too big for valbuf[20] ('\0' is the 21th character)." */
+    /* it was originally an array of [20] */
+    char valbuf[21];
     
     /* determine the value to print */
-    sprintf(valbuf,format,arg);
+    snprintf(valbuf,sizeof(valbuf),format,arg);
 
     /* print the field */
     printf("     ");
@@ -634,7 +765,7 @@ StatLineField(
     char valbuf[20];
     
     /* determine the value to print */
-    sprintf(valbuf,format,arg);
+    snprintf(valbuf,sizeof(valbuf),format,arg);
 
     /* print the field */
     printf("     ");
@@ -661,7 +792,7 @@ StatLineFieldF(
     
     /* determine the value to print */
     if (printable)
-	sprintf(valbuf,format,arg);
+	snprintf(valbuf,sizeof(valbuf),format,arg);
 
     /* print the field */
     printf("     ");
@@ -683,7 +814,7 @@ StatLineOne(
     char labbuf[20];
     
     /* format the label */
-    sprintf(labbuf, "%s:", label);
+    snprintf(labbuf,sizeof(labbuf), "%s:", label);
 
     /* print the field */
     printf("%-18s %9s %-5s", labbuf, value, units);
@@ -700,7 +831,7 @@ elapsed2str(
 
     etime_secs = etime / 1000000.0;
     etime_usecs = 1000000 * (etime/1000000.0 - (double)etime_secs);
-    sprintf(buf,"%lu:%02lu:%02lu.%06lu",
+    snprintf(buf,sizeof(buf),"%lu:%02lu:%02lu.%06lu",
 	    etime_secs / (60 * 60),
 	    etime_secs % (60 * 60) / 60,
 	    (etime_secs % (60 * 60)) % 60,
@@ -737,13 +868,9 @@ UDPPrintBrief(
 
     /* new version */
     fprintf(stdout,"%*s", -max_width, UDPFormatBrief(pup));
-#ifdef HAVE_LONG_LONG
-    fprintf(stdout," %4llu>", pab->packets);
-    fprintf(stdout," %4llu<", pba->packets);
-#else /* HAVE_LONG_LONG */
-    fprintf(stdout," %4lu>", pab->packets);
-    fprintf(stdout," %4lu<", pba->packets);
-#endif /* HAVE_LONG_LONG */
+
+    fprintf(stdout," %4" FS_ULL ">", pab->packets);
+    fprintf(stdout," %4" FS_ULL "<", pba->packets);
 
     fprintf(stdout,"\n");
 }
@@ -757,7 +884,7 @@ UDPFormatBrief(
     ucb *pba = &pup->b2a;
     static char infobuf[100];
 
-    sprintf(infobuf,"%s - %s (%s2%s)",
+    snprintf(infobuf,sizeof(infobuf),"%s - %s (%s2%s)",
 	    pup->a_endpoint, pup->b_endpoint,
 	    pab->host_letter, pba->host_letter);
     return(infobuf);
