@@ -1,34 +1,60 @@
 /*
- * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999
- *	Ohio University.  All rights reserved.
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+ *	Ohio University.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that: (1) source code
- * distributions retain the above copyright notice and this paragraph
- * in its entirety, (2) distributions including binary code include
- * the above copyright notice and this paragraph in its entirety in
- * the documentation or other materials provided with the
- * distribution, and (3) all advertising materials mentioning features
- * or use of this software display the following acknowledgment:
- * ``This product includes software developed by the Ohio University
- * Internetworking Research Laboratory.''  Neither the name of the
- * University nor the names of its contributors may be used to endorse
- * or promote products derived from this software without specific
- * prior written permission.
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * ---
+ * 
+ * Starting with the release of tcptrace version 6 in 2001, tcptrace
+ * is licensed under the GNU General Public License (GPL).  We believe
+ * that, among the available licenses, the GPL will do the best job of
+ * allowing tcptrace to continue to be a valuable, freely-available
+ * and well-maintained tool for the networking community.
+ *
+ * Previous versions of tcptrace were released under a license that
+ * was much less restrictive with respect to how tcptrace could be
+ * used in commercial products.  Because of this, I am willing to
+ * consider alternate license arrangements as allowed in Section 10 of
+ * the GNU GPL.  Before I would consider licensing tcptrace under an
+ * alternate agreement with a particular individual or company,
+ * however, I would have to be convinced that such an alternative
+ * would be to the greater benefit of the networking community.
+ * 
+ * ---
+ *
+ * This file is part of Tcptrace.
+ *
+ * Tcptrace was originally written and continues to be maintained by
+ * Shawn Ostermann with the help of a group of devoted students and
+ * users (see the file 'THANKS').  The work on tcptrace has been made
+ * possible over the years through the generous support of NASA GRC,
+ * the National Science Foundation, and Sun Microsystems.
+ *
+ * Tcptrace is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Tcptrace is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Tcptrace (in the file 'COPYING'); if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+ * MA 02111-1307 USA
  * 
  * Author:	Shawn Ostermann
  * 		School of Electrical Engineering and Computer Science
  * 		Ohio University
  * 		Athens, OH
  *		ostermann@cs.ohiou.edu
+ *		http://www.tcptrace.org/
  */
 static char const copyright[] =
-    "@(#)Copyright (c) 1999 -- Shawn Ostermann -- Ohio University.  All rights reserved.\n";
+    "@(#)Copyright (c) 2001 -- Ohio University.\n";
 static char const rcsid[] =
-    "@(#)$Header: /home/sdo/src/tcptrace/src/RCS/tcpdump.c,v 5.6 1999/08/30 15:52:51 sdo Exp $";
+    "@(#)$Header: /usr/local/cvs/tcptrace/tcpdump.c,v 5.16 2002/06/21 09:56:27 alakhian Exp $";
 
 #include <stdio.h>
 #include "tcptrace.h"
@@ -76,23 +102,26 @@ static int callback(
 
     /* kindof ugly, but about the only way to make them fit together :-( */
     switch (type) {
-      case DLT_EN10MB:
+      case 100:
+	/* for some reason, the windows version of tcpdump is using */
+	/* this.  It looks just like ethernet to me */
+      case PCAP_DLT_EN10MB:
 	memcpy(&eth_header,buf,EH_SIZE);  /* save ether header */
 	memcpy(ip_buf,buf+EH_SIZE,iplen);
 	callback_plast = (char *)ip_buf+iplen-EH_SIZE-1;
 	break;
-      case DLT_IEEE802:
+      case PCAP_DLT_IEEE802:
 	/* just pretend it's "normal" ethernet */
 	offset = 14;		/* 22 bytes of IEEE cruft */
 	memcpy(&eth_header,buf,EH_SIZE);  /* save ether header */
 	memcpy(ip_buf,buf+offset,iplen);
 	callback_plast = (char *)ip_buf+iplen-offset-1;
 	break;
-      case DLT_SLIP:
+      case PCAP_DLT_SLIP:
 	memcpy(ip_buf,buf+16,iplen);
 	callback_plast = (char *)ip_buf+iplen-16-1;
 	break;
-      case DLT_FDDI:
+      case PCAP_DLT_FDDI:
 	if (offset < 0)
 	      offset = find_ip_fddi(buf,iplen);
 	if (offset < 0)
@@ -100,28 +129,34 @@ static int callback(
 	memcpy((char *)ip_buf,buf+offset,iplen);
 	callback_plast = ip_buf+iplen-offset-1;
 	break;
-      case DLT_NULL:
+      case PCAP_DLT_NULL:
 	/* no phys header attached */
 	offset = 4;
 	memcpy((char *)ip_buf,buf+offset,iplen);
 	callback_plast = ip_buf+iplen-offset-1;
 	break;
-      case DLT_ATM_RFC1483:
+      case PCAP_DLT_ATM_RFC1483:
 	/* ATM RFC1483 - LLC/SNAP ecapsulated atm */
 	memcpy((char *)ip_buf,buf+8,iplen);
 	callback_plast = ip_buf+iplen-8-1;
 	break;
-      case DLT_RAW:
+      case PCAP_DLT_RAW:
 	/* raw IP */
 	offset = 0;
 	memcpy((char *)ip_buf,buf+offset,iplen);
 	callback_plast = ip_buf+iplen-offset-1;
 	break;
+      case PCAP_DLT_LINUX_SLL:
+	/* linux cooked socket */
+	offset = 16;
+	memcpy((char *)ip_buf, buf+offset, iplen);
+	callback_plast = ip_buf+iplen-offset-1;
+	break;
       default:
 	fprintf(stderr,"Don't understand link-level format (%d)\n", type);
 	exit(1);
     }
-
+    
     return(0);
 }
 
@@ -176,7 +211,9 @@ pread_tcpdump(
 	*pphystype = PHYS_ETHER; /* everything assumed to be ethernet */
 	*ppip      = (struct ip *) ip_buf;
 	*pplast    = callback_plast; /* last byte in IP packet */
-	*ptime     = callback_phdr->ts;
+	/* (copying time structure in 2 steps to avoid RedHat brain damage) */
+	ptime->tv_usec = callback_phdr->ts.tv_usec;
+	ptime->tv_sec = callback_phdr->ts.tv_sec;
 	*plen      = callback_phdr->len;
 	*ptlen     = callback_phdr->caplen;
 
@@ -193,13 +230,17 @@ pread_tcpdump(
 }
 
 
-pread_f *is_tcpdump(void)
+pread_f *is_tcpdump(char *filename)
 {
     char errbuf[100];
     char *physname = "<unknown>";
     int type;
 
-    if ((pcap = pcap_open_offline("-",errbuf)) == NULL) {
+#ifdef __WIN32   
+      if ((pcap = pcap_open_offline(filename, errbuf)) == NULL) {
+#else       
+      if ((pcap = pcap_open_offline("-", errbuf)) == NULL) {
+#endif /* __WIN32 */	  
 	if (debug > 2)
 	    fprintf(stderr,"PCAP said: '%s'\n", errbuf);
 	rewind(stdin);
@@ -210,48 +251,51 @@ pread_f *is_tcpdump(void)
     if (debug) {
 	printf("Using 'pcap' version of tcpdump\n");
 	if (debug > 1) {
-	    struct pcap_file_header *pfhdr = (void *)pcap;
-	    printf("\tpcap->magic: %d\n", pfhdr->magic);
-	    printf("\tpcap->version_major: %d\n", pfhdr->version_major);
-	    printf("\tpcap->version_minor: %d\n", pfhdr->version_minor);
-	    printf("\tpcap->thiszone: %d\n", pfhdr->thiszone);
-	    printf("\tpcap->sigfigs: %d\n", pfhdr->sigfigs);
-	    printf("\tpcap->snaplen: %d\n", pfhdr->snaplen);
-	    printf("\tpcap->linktype: %d\n", pfhdr->linktype);
+	    printf("\tversion_major: %d\n", pcap_major_version(pcap));
+	    printf("\tversion_minor: %d\n", pcap_minor_version(pcap));
+	    printf("\tsnaplen: %d\n", pcap_snapshot(pcap));
+	    printf("\tlinktype: %d\n", pcap_datalink(pcap));
+	    printf("\tswapped: %d\n", pcap_is_swapped(pcap));
 	}
     }
 
     /* check the phys type (pretend everything is ethernet) */
     memset(&eth_header,0,EH_SIZE);
     switch (type = pcap_datalink(pcap)) {
-      case DLT_EN10MB:
+case 100:
+      case PCAP_DLT_EN10MB:
 	/* OK, we understand this one */
 	physname = "Ethernet";
 	break;
-      case DLT_IEEE802:
+      case PCAP_DLT_IEEE802:
 	/* just pretend it's normal ethernet */
 	physname = "Ethernet";
 	break;
-      case DLT_SLIP:
+      case PCAP_DLT_SLIP:
 	eth_header.ether_type = htons(ETHERTYPE_IP);
 	physname = "Slip";
 	break;
-      case DLT_FDDI:
+      case PCAP_DLT_FDDI:
 	eth_header.ether_type = htons(ETHERTYPE_IP);
 	physname = "FDDI";
 	break;
-      case DLT_NULL:
+      case PCAP_DLT_NULL:
 	eth_header.ether_type = htons(ETHERTYPE_IP);
 	physname = "NULL";
 	break;
-      case DLT_ATM_RFC1483:
+      case PCAP_DLT_ATM_RFC1483:
 	eth_header.ether_type = htons(ETHERTYPE_IP);
 	physname = "ATM, LLC/SNAP encapsulated";
 	break;
-      case DLT_RAW:
+      case PCAP_DLT_RAW:
 	eth_header.ether_type = htons(ETHERTYPE_IP);
 	physname = "RAW_IP";
 	break;
+      case PCAP_DLT_LINUX_SLL:
+	/* linux cooked socket type */
+	eth_header.ether_type = htons(ETHERTYPE_IP);
+	physname = "Linux Cooked Socket";
+	break;
       default:
 	if (debug)
 	    fprintf(stderr,"is_tcpdump: I think it's tcpdump, but I don't understand link format (%d)\n", type);
@@ -279,7 +323,7 @@ PcapSavePacket(
     struct ip *pip,
     void *plast)
 {
-    static FILE *f_savefile = NULL;
+    static MFILE *f_savefile = NULL;
     struct pcap_pkthdr phdr;
     int wlen;
 
@@ -287,7 +331,7 @@ PcapSavePacket(
 	struct pcap_file_header fhdr;
 
 	/* try to open the file */
-	if ((f_savefile = fopen(filename, "w")) == NULL) {
+	if ((f_savefile = Mfopen(filename, "w")) == NULL) {
 	    perror(filename);
 	    exit(-1);
 	}
@@ -300,33 +344,35 @@ PcapSavePacket(
 
 	fhdr.thiszone = 0;	/* don't have this info, just make it up */
 	fhdr.snaplen = 1000000;	/* don't have this info, just make it up */
-	fhdr.linktype = DLT_EN10MB; /* always Ethernet (10Mb) */
+	fhdr.linktype = PCAP_DLT_EN10MB; /* always Ethernet (10Mb) */
 	fhdr.sigfigs = 0;
 
 	/* write the header */
-	fwrite((char *)&fhdr, sizeof(fhdr), 1, f_savefile);
+	Mfwrite((char *)&fhdr, sizeof(fhdr), 1, f_savefile);
 
 	if (debug)
 	    fprintf(stderr,"Created pcap save file '%s'\n", filename);
     }
 
     /* create the packet header */
-    phdr.ts = current_time;
-    phdr.caplen = (unsigned)plast - (unsigned)pip + 1;
+    /* (copying time structure in 2 steps to avoid RedHat brain damage) */
+    phdr.ts.tv_sec = current_time.tv_sec;
+    phdr.ts.tv_usec = current_time.tv_usec;
+    phdr.caplen = (char *)plast - (char *)pip + 1;
     phdr.caplen += EH_SIZE;	/* add in the ether header */
     phdr.len = EH_SIZE + ntohs(PIP_LEN(pip));	/* probably this */
 
     /* write the packet header */
-    fwrite(&phdr, sizeof(phdr), 1, f_savefile);
+    Mfwrite(&phdr, sizeof(phdr), 1, f_savefile);
 
     /* write a (bogus) ethernet header */
     memset(&eth_header,0,EH_SIZE);
     eth_header.ether_type = htons(ETHERTYPE_IP);
-    fwrite(&eth_header, sizeof(eth_header), 1, f_savefile);
+    Mfwrite(&eth_header, sizeof(eth_header), 1, f_savefile);
 
     /* write the IP/TCP parts */
     wlen = phdr.caplen - EH_SIZE;	/* remove the ether header */
-    fwrite(pip, wlen, 1, f_savefile);
+    Mfwrite(pip, wlen, 1, f_savefile);
 }
     
 
